#include "driver.h"
#include "dispatchcreateclose.h"

NTSTATUS
DispatchCreate(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )
/*
Routine Description:
     Dit is de driveringang voor CreateFile aanroepen vanuit User Mode applicaties
     (applicaties kunnen "\\.\d-x\yyzz" openen,
     waarbij d de apparaatnaam is, yy is het interface nummer en zz het endpoint adres).

     Dit is de plek waar code toegevoegd zou worden wanneer er symbolische links tussen endpoints
     (bijvoorbeeld pipes in het geval van USB software) en User Mode bestandsnamen.
     De naamgeving van deze links is helemaal vrij,
     hoewel de bovenstaande naamgeving een manier biedt om bronnen binnen een Device aan te duiden
     via de bekende bestand- en mappennaamgeving.
Arguments:
    DeviceObject - pointer naar het Device Object voor deze instantie van het USB device
Return Value:
    NT status code = STATUS_SUCCESS
*/
{
   PDEVICE_EXTENSION pDeviceExtension = (PDEVICE_EXTENSION)DeviceObject->DeviceExtension;
   KdPrint((debugPrefix "DispatchCreate: begin\n"));

   if (!pDeviceExtension->Started)
      return STATUS_UNSUCCESSFUL;

   // vermeerder de open handle teller
   pDeviceExtension->OpenHandles++;

   // creer hier alle Symbolic Links

   Irp->IoStatus.Status = STATUS_SUCCESS;
   Irp->IoStatus.Information = 0;

   IoCompleteRequest(Irp,
                     IO_NO_INCREMENT
                     );

   KdPrint((debugPrefix "DispatchCreate: einde\n"));
   return STATUS_SUCCESS;
} // DispatchCreate

NTSTATUS
DispatchClose(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )
/*
Routine Description:
    Ingang voor CloseHandle aanroepen vanuit User Mode applicaties om handles te sluiten die geopend waren
Arguments:
    DeviceObject - pointer naar het Device Object voor deze instantie van het USB device
    Irp          - pointer naar een irp
Return Value:
    NT status code = STATUS_SUCCESS
*/
{
   PDEVICE_EXTENSION pDeviceExtension = (PDEVICE_EXTENSION)DeviceObject->DeviceExtension;
   KdPrint((debugPrefix "DispatchClose: begin\n"));

   // verminder de open handle teller
   pDeviceExtension->OpenHandles--;

   Irp->IoStatus.Status = STATUS_SUCCESS;
   Irp->IoStatus.Information = 0;

   IoCompleteRequest(Irp,
                     IO_NO_INCREMENT
                     );

   KdPrint((debugPrefix "DispatchClose: einde\n"));
   return STATUS_SUCCESS;
} // DispatchClose
