#include "driver.h"
#include "dispatchpower.h"

NTSTATUS
DispatchPower(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP           Irp
    )
/*
Routine Description:
   Verwerk de IRPs die naar dit apparaat worden verzonden.

Arguments:
   DeviceObject - pointer naar een Device Object
   Irp - pointer naar een I/O Request Packet

Return Value:
   NTSTATUS
*/
{
   PIO_STACK_LOCATION irpStack, nextStack;
   PDEVICE_EXTENSION pDeviceExtension = (PDEVICE_EXTENSION)DeviceObject->DeviceExtension;
   NTSTATUS ntStatus;
   KdPrint((debugPrefix "DispatchPower: begin\n"));

   Irp->IoStatus.Status = STATUS_SUCCESS;
   Irp->IoStatus.Information = 0;

   // verkrijg een pointer naar de huidige stack locatie in de IRP.
   // dat is de plek waar de functiecodes en parameters zich bevinden.
   irpStack = IoGetCurrentIrpStackLocation(Irp);

   KdPrint((debugPrefix "DispatchPower: IRP_MJ_POWER MIN=0x%x Type=0x%x State=0x%x\n",irpStack->MinorFunction,
      irpStack->Parameters.Power.Type,
      irpStack->Parameters.Power.State.DeviceState));

   switch (irpStack->MinorFunction)
   {
      case IRP_MN_SET_POWER:
      {
         switch (irpStack->Parameters.Power.Type)
         {
            case SystemPowerState:
               break;

            case DevicePowerState:
               switch (irpStack->Parameters.Power.State.DeviceState)
               {
                  case PowerDeviceD3:
                     KdPrint((debugPrefix "DispatchPower: IRP_MN_SET_D3\n"));
                     break;
                  case PowerDeviceD2:
                     KdPrint((debugPrefix "DispatchPower: IRP_MN_SET_D2\n"));
                     break;
                  case PowerDeviceD1:
                     KdPrint((debugPrefix "DispatchPower: IRP_MN_SET_D1\n"));
                     break;
                  case PowerDeviceD0:
                     KdPrint((debugPrefix "DispatchPower: IRP_MN_SET_D0\n"));
                     break;
                }
                break;
         }
         break;
      }
      case IRP_MN_QUERY_POWER:
      {
         // bepaal welk type Power aanvraag dit is
         switch (irpStack->Parameters.Power.Type)
         {
            case SystemPowerState:
               break;

            case DevicePowerState:
               switch (irpStack->Parameters.Power.State.DeviceState)
               {
                  case PowerDeviceD2:
                  {
                     KdPrint((debugPrefix "DispatchPower: IRP_MN_QUERY_D2\n"));
                     break;
                  }
                  case PowerDeviceD1:
                  {
                     KdPrint((debugPrefix "DispatchPower: IRP_MN_QUERY_D1\n"));
                     break;
                  }
                  case PowerDeviceD3:
                  {
                     KdPrint((debugPrefix "DispatchPower: IRP_MN_QUERY_D3\n"));
                     break;
                  }
               }
               break;
         }
         break;
      }
      default:
         // een Power MinorFunction is niet afgehandeld
         KdPrint((debugPrefix "DispatchPower: Power MinorFunction niet afgehandeld\n"));
   }

   nextStack = IoGetNextIrpStackLocation(Irp);
   ASSERT(nextStack != NULL);
   RtlCopyMemory(nextStack, irpStack, sizeof(IO_STACK_LOCATION));

   // alle PNP_POWER berichten worden doorgegeven aan het StackDeviceObject dat
   // meegegeven was binnen AddDevice.

   // This stack device object is managed by the USB software subsystem,
   // and so this IRP must be propagated to the owning device driver for
   // that stack device object, so that driver in turn can perform any
   // device state management (e.g., remove its device object, etc.).
   KdPrint((debugPrefix "DispatchPower: Power IRP wordt naar beneden doorgegeven\n"));

   PoStartNextPowerIrp(Irp);
   ntStatus = PoCallDriver(pDeviceExtension->StackDeviceObject, Irp);

   // If lower layer driver marked the Irp as pending then reflect that by
   // calling IoMarkIrpPending.
   if (ntStatus == STATUS_PENDING)
   {
      IoMarkIrpPending(Irp);
      KdPrint((debugPrefix "DispatchPower: Power IRP is teruggekomen met STATUS_PENDING (%x)\n", ntStatus));
   }
   else 
      KdPrint((debugPrefix "DispatchPower: Power IRP is teruggekomen, status = %x\n", ntStatus));

   KdPrint((debugPrefix "DispatchPower: einde (ntStatus = %x)\n", ntStatus));
   return ntStatus;
} // DispatchPower
