/////////////////////////////////////////////////////////////////////////////
//
//		hoofddriverbestand
//
/////////////////////////////////////////////////////////////////////////////

#include "driver.h"
#include "utility.h"
#include "usb.h"
#include "usbgetdescriptor.h"
#include "usbrequests.h"
#include "usbezusb.h"
#include "dispatchcreateclose.h"
#include "dispatchioctl.h"
#include "dispatchpnp.h"
#include "dispatchpower.h"

VOID
Unload(
    IN PDRIVER_OBJECT DriverObject
    )
/*
Routine Description:
    Geeft alle gealloceerde bronnen vrij. Wordt niet uitgevoerd als Windows wordt afgesloten.
    TODO: hier moet de driverauteur code toevoegen die uitgevoerd wordt bij het vrijgeven van de driver

Arguments:
    DriverObject - pointer naar een Driver Object

Return Value:
    geen
*/
{
   KdPrint((debugPrefix "Unload: begin\n"));

   // TODO: geef globale bronnen vrij die gealloceerd zijn brinnen DriverEntry
    
   KdPrint((debugPrefix "Unload: einde\n"));
}

NTSTATUS
CreateDeviceObject(
    IN PDRIVER_OBJECT DriverObject,
    IN PDEVICE_OBJECT *DeviceObject,
    LONG Instance
    )
/*
Routine Description:
    Maakt een Functional Device Object aan. Wordt aangeroepen door de AddDevice routine.

Arguments:
    DriverObject - pointer naar het Driver Object voor het device
    DeviceObject - pointer om het aangemaakte Device Object terug te geven.
    Instance - instantie van het aan te maken device.

Return Value:
    STATUS_SUCCESS if successful,
    STATUS_UNSUCCESSFUL otherwise
*/
{
   NTSTATUS ntStatus;
   WCHAR deviceLinkBuffer[]  = L"\\DosDevices\\Ezusb-0";
   UNICODE_STRING deviceLinkUnicodeString;
   WCHAR deviceNameBuffer[]  = L"\\Device\\Ezusb-0";
   UNICODE_STRING deviceNameUnicodeString;
   PDEVICE_EXTENSION pDeviceExtension;
   STRING deviceName;

   KdPrint((debugPrefix "CreateDeviceObject: begin (instance = %d)\n", Instance));

   // maak de devicenamen kloppend op basis van Instance
   deviceLinkBuffer[18] = (USHORT)('0' + Instance);
   deviceNameBuffer[14] = (USHORT)('0' + Instance);

   KdPrint((debugPrefix "CreateDeviceObject: kernelmode devicenaam wordt aangemaakt (%ws)\n", deviceNameBuffer));

   RtlInitUnicodeString(&deviceNameUnicodeString, deviceNameBuffer);

   // geef de unicode string weer
   // OPMERKING: de string moet eerst geconverteerd worde naar Unicode vanwege een bug in de
   //            Debugger die ervoor zorgt dat Unicode Strings niet geprint kunnen worden naar 
   //            het debug device
   deviceName.Buffer = NULL;

   ntStatus = RtlUnicodeStringToAnsiString(&deviceName,
                                           &deviceNameUnicodeString, 
                                           TRUE);

   if (NT_SUCCESS(ntStatus))
   {
      KdPrint((debugPrefix "CreateDeviceObject: kernelmode devicenaam is aangemaakt (%s)\n", deviceName.Buffer));
      RtlFreeAnsiString (&deviceName);
   }
   else
      KdPrint((debugPrefix "CreateDeviceObject: unicode naar AnsiString conversie mislukt (ntStatus = 0x%x)\n",ntStatus));

   ntStatus = IoCreateDevice (DriverObject,
                              sizeof(DEVICE_EXTENSION),
                              &deviceNameUnicodeString,
                              FILE_DEVICE_UNKNOWN,
                              0,
                              FALSE,
                              DeviceObject);

   if (NT_SUCCESS(ntStatus))
   {
      // initialiseer de device extensie
      pDeviceExtension = (PDEVICE_EXTENSION)((*DeviceObject)->DeviceExtension);

      RtlCopyMemory(pDeviceExtension->DeviceLinkNameBuffer,
                    deviceLinkBuffer,
                    sizeof(deviceLinkBuffer));

      pDeviceExtension->OpenHandles = 0;
      pDeviceExtension->ConfigurationHandle = NULL;
      pDeviceExtension->DeviceDescriptor = NULL;
      pDeviceExtension->NeedCleanup = FALSE;
      pDeviceExtension->Started = FALSE;

      // initialiseer de interface
      pDeviceExtension->Interface = NULL;

      RtlInitUnicodeString (&deviceLinkUnicodeString, deviceLinkBuffer);
      KdPrint((debugPrefix "CreateDeviceObject: Symbolic Link naam aanmaken (%ws)\n", deviceLinkBuffer));

      ntStatus = IoCreateSymbolicLink(&deviceLinkUnicodeString, &deviceNameUnicodeString);
   }

   KdPrint((debugPrefix "CreateDeviceObject: einde (ntStatus = %x)\n", ntStatus));

   return ntStatus;
}

NTSTATUS
AddDevice(
    IN PDRIVER_OBJECT DriverObject,
    IN PDEVICE_OBJECT PhysicalDeviceObject
    )
/*
Routine Description:
    Deze routine wordt aangeroepen om een nieuwe device instantie aan te maken
Arguments:
    DriverObject - pointer naar het Driver Object voor deze instantie van het apparaat
    PhysicalDeviceObject - pointer naar een Fysieke Device Object (PDO) aangemaakt door de bus
Return Value:
    STATUS_SUCCESS indien succesvol,
    anders STATUS_UNSUCCESSFUL
*/
{
   NTSTATUS          ntStatus = STATUS_SUCCESS;
   PDEVICE_OBJECT    deviceObject = NULL;
   PDEVICE_EXTENSION pDeviceExtension;
   int               instance = 0;
   KdPrint((debugPrefix "AddDevice: begin\n"));

   // MAX_EZUSB_DEVICES is het aantal te controleren instantienummers voordat het
   // zoeken naar een vrij instantienummer gestaakt wordt. het kan mogelijk de
   // performance ten goede komen als er niet tevergeefs bijvoorbeeld 100
   // instantienummers worden onderzocht.
#define MAX_USB_DEVICES 8

   // creer het Functionele Device Object (FDO).
   // de driver ondersteunt meerdere USB apparaten.
   // deze lus zal een beschikbaar instantienummer zoeken.
   // het instantienummer blijft worden vermeerderd totdat
   // een aanroep aan CreateDeviceObject slaagt of
   // het maximum aantal te onderzoeken instantienummers is bereikt.
   do
   {
      ntStatus = CreateDeviceObject(DriverObject, &deviceObject, instance);
      instance++;
   } while (!NT_SUCCESS(ntStatus) && (instance < MAX_USB_DEVICES));

   if (NT_SUCCESS(ntStatus))
   {
      pDeviceExtension = (PDEVICE_EXTENSION)deviceObject->DeviceExtension;

      // niet plug-en-play drivers maken gewoonlijk een Device Object aan
      // brinnen DriverEntry driver entry. In dat geval wist de I/O manager de vlag uit.
      // Omdat we nu zelf een Device Object aanmaken ten gevolge van
      // een PnP START_DEVICE IRP, moeten we deze vlag zelf uitwissen.
      deviceObject->Flags &= ~DO_DEVICE_INITIALIZING;

      // de driver gebruikt direct I/O voor read/write aanvragen
      deviceObject->Flags |= DO_DIRECT_IO;
      deviceObject->Flags |= DO_POWER_PAGABLE;

      // sla het Physical Device Object op
      pDeviceExtension->PhysicalDeviceObject = PhysicalDeviceObject;

      // koppel het Functional Device Object aan de stack.
      // dit is het Device Object dat gebruikt wordt om IRP's en URB's
      // van de USB software stack af te laten dalen
      pDeviceExtension->StackDeviceObject =
         IoAttachDeviceToDeviceStack(deviceObject, PhysicalDeviceObject);

      ASSERT(pDeviceExtension->StackDeviceObject != NULL);

      pDeviceExtension->LastFailedUrbStatus = 0;

      pDeviceExtension->usage = 1; // gelockt tot RemoveDevice wordt uitgevoerd
      KeInitializeEvent(&pDeviceExtension->evRemove,
                        NotificationEvent,
                        FALSE); // "set when use count drops to zero"
   }

   KdPrint((debugPrefix "AddDevice: einde (ntStatus = %x)\n", ntStatus));
   return ntStatus;
}

NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    )
/*
Routine Description:
   Installeerbare driverinitialisatieingang.
   Hier wordt de driver aangeroepen wanneer deze door het I/O systeem wordt ingeladen.
   Deze ingang wordt direct aangeroepen door het I/O systeem.
Arguments:
   DriverObject - pointer naar het Driver Object
   RegistryPath - pointer naar een Unicode string die het pad aangeeft naar
                  de driverspecifieke sleutel in het register
Return Value:
   STATUS_SUCCESS indien succesvol,
   anders STATUS_UNSUCCESSFUL
*/
{
   NTSTATUS ntStatus = STATUS_SUCCESS;
   PDEVICE_OBJECT deviceObject = NULL;
   KdPrint((debugPrefix "DriverEntry: begin (build = %s/%s)\n", __DATE__, __TIME__));

   // creer dispatch "aanknopingspunten" voor de verschillende gebeurtenissen
   // die door deze driver worden afgehandeld.
   // Bijvoorbeeld device I/O control aanroepen
   // (wanneer een user mode applicatie de DeviceIoControl functie aanroept)
   // worden gedispatcht naar de routine naast IRP_MJ_DEVICE_CONTROL.

   DriverObject->DriverUnload = Unload;
   DriverObject->DriverExtension->AddDevice = AddDevice;

   // vul het MultiFunction array met pointers naar de dispatch functies
   DriverObject->MajorFunction[IRP_MJ_CREATE]         = DispatchCreate;
   DriverObject->MajorFunction[IRP_MJ_CLOSE]          = DispatchClose;
   DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = DispatchDeviceControl;
   DriverObject->MajorFunction[IRP_MJ_PNP]            = DispatchPnp;
   DriverObject->MajorFunction[IRP_MJ_POWER]          = DispatchPower;

   KdPrint((debugPrefix "DriverEntry: einde (ntStatus = %x)\n", ntStatus));
   return ntStatus;
}

///////////////////////////////////////////////////////////////////////////////
// @func Handle completion of a request by a lower-level driver
// @parm Functional device object
// @parm I/O request which has completed
// @parm Context argument supplied to IoSetCompletionRoutine, namely address of
// KEVENT object on which ForwardAndWait is waiting
// @comm This is the completion routine used for requests forwarded by ForwardAndWait. It
// sets the event object and thereby awakens ForwardAndWait.
// @comm Note that it's *not* necessary for this particular completion routine to test
// the PendingReturned flag in the IRP and then call IoMarkIrpPending. You do that in many
// completion routines because the dispatch routine can't know soon enough that the
// lower layer has returned STATUS_PENDING. In our case, we're never going to pass a
// STATUS_PENDING back up the driver chain, so we don't need to worry about this.

NTSTATUS 
IoCompletion(
   IN PDEVICE_OBJECT DeviceObject,
   IN PIRP Irp,
   IN PVOID Context
   )
/*
Routine Description:
   Verwerkt completering van een verzoek van een lagerliggende driver af
Arguments:
   DeviceObject - Functional Device Object (FDO)
   Irp - I/O request dat compleet is
   Context - context parameter meegegeven aan IoSetCompletionRoutine:
             het adres van het KEVENT object waarop UtilForwardAndWait aan het wachten is
Return Value:
   STATUS_MORE_PROCESSING_REQUIRED
*/
{
   KeSetEvent((PKEVENT)Context, 0, FALSE);
   return STATUS_MORE_PROCESSING_REQUIRED;
}
