/////////////////////////////////////////////////////////////////////////////
//
//		hoofddriverheaderbestand
//
/////////////////////////////////////////////////////////////////////////////

#ifndef DRIVER_H
#define DRIVER_H

// includebestanden die nodig zijn voor WDM driverondersteuning
#include <wdm.h>
#include "stdio.h"
#include "stdarg.h"

// includebestanden die nodig zijn voor USB ondersteuning
#include "usbdi.h"
#include "usbdlib.h"

#ifndef _BYTE_DEFINED
#define _BYTE_DEFINED
typedef unsigned char BYTE;
#endif // !_BYTE_DEFINED

#ifndef _WORD_DEFINED
#define _WORD_DEFINED
typedef unsigned short WORD;
#endif // !_WORD_DEFINED

// deze informatie moet gelijkblijven aan de informatie in ezusbsys.rc
#define EZUSB_MAJOR_VERSION		01
#define EZUSB_MINOR_VERSION		00
#define EZUSB_BUILD_VERSION		0000

// de maximale lengte 
#define USB_NAME_MAX 64

#define debugPrefix "EZUSB.SYS: "

// het interfacenummer op het apparaat dat de driver verwacht te gaan gebruiken.
// dit komt in het bInterfaceNumber veld van de Interface Descriptor te staan,
// vandaar dat de device driver deze waarde nodig heeft
#define SAMPLE_INTERFACE_NBR 0x00

// een structure die de instantieinformatie behorende bij dit apparaat representeert
typedef struct _DEVICE_EXTENSION
{

   // Physical Device Object
   PDEVICE_OBJECT PhysicalDeviceObject;
   // Device Object dat aangeroepen kan worden wanneer er URB's/IRP's naar de USB stack verzonden worden
   PDEVICE_OBJECT StackDeviceObject;
   // geeft aan dat er een "STOP" bericht ontvangen is
   BOOLEAN Stopped;
   // geeft aan dat het apparaat genumereerd is en geconfigureerd.
   // wordt gebruikt om verzoeken op te houden totdat de driver er klaar voor is
   BOOLEAN Started;
   // geeft aan of het device opgeschoond moet worden
   // (bijvoorbeeld wat configuratie die heeft plaatsgevonden en ongedaan gemaakt moet worden)
   BOOLEAN NeedCleanup;
   // configuratie handle voor de configuratie waarin het device zich op dit moment verkeert
   USBD_CONFIGURATION_HANDLE ConfigurationHandle;
   // pointer naar de USB Device Descriptor voor dit device
   PUSB_DEVICE_DESCRIPTOR DeviceDescriptor;
   // er wordt maximaal n interface ondersteund
   PUSBD_INTERFACE_INFORMATION Interface;
   // het aantal open device handles bij dit Device Object
   // wordt vermeerderd door Create en verminderd door Close
   ULONG OpenHandles;
   // naambuffer vaoor de Functional Device Object linknaam
   WCHAR DeviceLinkNameBuffer[USB_NAME_MAX];
   // wordt gebruikt om de URB status op te slaan van de meest recent gefaalde URB.
   // als een USB overdracht mislukt kan een aanroeper IOCTL_EZUSB_GET_LAST_ERROR
   // gebruiken om deze waarde te ontvangen.
   // er is slechts ruimte voor n stuk, dus het aanvragen hiervan moet snel gebeuren
   // (in ieder geval voordat de volgende URB fout plaatsheeft)
   USBD_STATUS LastFailedUrbStatus;
   // gebruiksteller voor het device. wordt vermeerderd wanneer de driver een verzoek
   // ontvangt en verminderd wanneer een verzoek gecompleteerd is.
   LONG usage;
   // dit event wordt gezet/gesignaleerd wanneer het toegestaan is het device te verwijderen
   KEVENT evRemove;
   // TRUE wanneer er geprobeerd wordt het device te verwijderen
   BOOLEAN removing;
} DEVICE_EXTENSION, *PDEVICE_EXTENSION;

// deze structure wordt niet gebruikt binnen deze driver,
// maar kan gebruikt worden wanneer de driver wordt uitgebreid om om te gaan met USB pipes,
// zodat de structure als voorbeeld of model gebruikt kan worden.
typedef struct _USB_PIPE {
   ULONG Mode;
   ULONG Option;
   ULONG Param1;
   ULONG Param2;
   WCHAR Name[USB_NAME_MAX];
   PUSBD_PIPE_INFORMATION PipeInfo;
} EZUSB_PIPE, *PEZUSB_PIPE;

typedef struct _BULK_LATENCY_CONTROL
{
   ULONG bulkPipeNum;
   ULONG intPipeNum;
   ULONG loops;
} BULK_LATENCY_CONTROL, *PBULK_LATENCY_CONTROL;

typedef struct _EZUSB_FIRMWARE
{
   // tag contains a string to identify the start of the firmware
   // image in the driver binary.  Another utilty can then be used
   // to replace the firmware image inthe driver without requiring
   // a recompile
   UCHAR tag[10];
   ULONG size;
   UCHAR firmware[];
} EZUSB_FIRMWARE, *PEZUSB_FIRMWARE;

NTSTATUS 
IoCompletion(
   IN PDEVICE_OBJECT DeviceObject,
   IN PIRP Irp,
   IN PVOID Context
   );

#endif // DRIVER_H
