#include "driver.h"
#include "utility.h"

#define BYTES_PER_LINE 0x10

void
UtilDumpBuffer(PVOID pvBuffer, ULONG length)
{
    int    nItems    = 0;
    char   temp[64]  = "";
    char   temp2[64] = "";
	ULONG  i;
	ULONG  j;
	PUCHAR ptr = (PUCHAR)pvBuffer;

	for (i = 0; i < ((length + BYTES_PER_LINE - 1) / BYTES_PER_LINE); i++)
	{
		sprintf(temp, "%04X ", i * BYTES_PER_LINE);
		for (j = 0; j < BYTES_PER_LINE; j++)
		{
			if (((i * BYTES_PER_LINE) + j) < length)
			{
				sprintf(temp2, "%02X ", *ptr++);
				strcat(temp, temp2);
			}
		}
//	    SendMessage (hOutputBox, LB_ADDSTRING, 0, (LPARAM)temp);
        KdPrint((debugPrefix "%s\n",temp));
	}
}

///////////////////////////////////////////////////////////////////////////////
// @func Lock a SIMPLE device object
// @parm Address of our device extension
// @rdesc TRUE if it was possible to lock the device, FALSE otherwise.
// @comm A FALSE return value indicates that we're in the process of deleting
// the device object, so all new requests should be failed

BOOLEAN UtilLockDeviceObject(
   IN PDEVICE_OBJECT DeviceObject
   )
{
   PDEVICE_EXTENSION pDeviceExtension = (PDEVICE_EXTENSION)DeviceObject->DeviceExtension;

   // Increment use count on our device object
   LONG usage = InterlockedIncrement(&pDeviceExtension->usage);

   // AddDevice initialized the use count to 1, so it ought to be bigger than
   // one now. HandleRemoveDevice sets the "removing" flag and decrements the
   // use count, possibly to zero. So if we find a use count of "1" now, we
   // should also find the "removing" flag set.

   ASSERT((usage > 1) || pDeviceExtension->removing);

   // If device is about to be removed, restore the use count and return FALSE.
   // If we're in a race with HandleRemoveDevice (maybe running on another CPU),
   // the sequence we've followed is guaranteed to avoid a mistaken deletion of
   // the device object. If we test "removing" after HandleRemoveDevice sets it,
   // we'll restore the use count and return FALSE. In the meantime, if
   // HandleRemoveDevice decremented the count to 0 before we did our increment,
   // its thread will have set the remove event. Otherwise, we'll decrement to 0
   // and set the event. Either way, HandleRemoveDevice will wake up to finish
   // removing the device, and we'll return FALSE to our caller.
   // 
   // If, on the other hand, we test "removing" before HandleRemoveDevice sets it,
   // we'll have already incremented the use count past 1 and will return TRUE.
   // Our caller will eventually call UnlockDevice, which will decrement the use
   // count and might set the event HandleRemoveDevice is waiting on at that point.

   if (pDeviceExtension->removing)
   {
      if (InterlockedDecrement(&pDeviceExtension->usage) == 0)
         KeSetEvent(&pDeviceExtension->evRemove, 0, FALSE);
      return FALSE;
   }
   return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
// @func Unlock a SIMPLE device object
// @parm Address of our device extension
// @comm If the use count drops to zero, set the evRemove event because we're
// about to remove this device object.

void UtilUnlockDeviceObject(
   PDEVICE_OBJECT DeviceObject
   )
{
   PDEVICE_EXTENSION pDeviceExtension = (PDEVICE_EXTENSION)DeviceObject->DeviceExtension;
   LONG usage = InterlockedDecrement(&pDeviceExtension->usage);

   ASSERT(usage >= 0);

   if (usage == 0)
   { // removing device
      ASSERT(pDeviceExtension->removing); // HandleRemoveDevice should already have set this
      KeSetEvent(&pDeviceExtension->evRemove, 0, FALSE);
   } // removing device
}

NTSTATUS
UtilCompleteRequest(
   IN PIRP Irp,
   IN NTSTATUS status,
   IN ULONG info
   )
/*
Routine Description:
   Markeer een I/O request als compleet
Arguments:
   Irp - het specifieke I/O request
   status - standaard statuscode
   info - aanvullende informatie i.v.m. statuscode
Return Value:
   STATUS_SUCCESS indien succesvol,
   anders STATUS_UNSUCCESSFUL
*/
{
   Irp->IoStatus.Status = status;
   Irp->IoStatus.Information = info;
   IoCompleteRequest(Irp, IO_NO_INCREMENT);
   return status;
}
